/* A constraint in the elaborated network.

Metropolis: Design Environment for Heterogeneus Systems.

Copyright (c) 1998-2004 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software and that appropriate acknowledgments are made
to the research of the Metropolis group.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/


package metropolis.metamodel.runtime;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;

//////////////////////////////////////////////////////////////////////////
//// Constraint
/**
 *  A constraint in the elaborated network.
 *
 *  @author   Xi Chen
 *  @version  $Id: Constraint.java,v 1.28 2004/09/07 15:02:23 cxh Exp $
 */
public class Constraint implements Serializable {

    ///////////////////////////////////////////////////////////////////
    ////                           constructors                    ////

    /** Build a new constraint in the network.
     *  @param kind  the constraint type, LTL, LOC or ELOC.
     */
    public Constraint(int kind) {
        this(null, kind, -1);
    }

    /** Build a new constraint in the network.
     *  @param container  the node which contains this constraint.
     *  @param kind  the constraint type, LTL, LOC or ELOC.
     *  @param index the sequence number of the constraint.
     */
    public Constraint(Object container, int kind, int index) {
        _kind       = kind;
        _index      = index;
        _source     = new String();

        if (_kind < LTL || _kind > LTLSYNCHIMPLY) {
            throw new RuntimeException("kind '" + _kind + "'was out of range, "
                    + "it must be >= " + LTL + " and < " + LTLSYNCHIMPLY);

        }

        try {
            if (container == null) {
                _container = null;
            } else {
                _container = Network.net.getNode(container);
            }
        } catch (Exception ex) {
            throw new RuntimeException("Problem creating a Constraint "
                    + "of kind " + kind + ", index " + index, ex);
        }
    }

    ///////////////////////////////////////////////////////////////////
    ////

    /** Kind for ltl constraint. */
    public static final int LTL   = 0x1;

    /** Kind for loc constraint. */
    public static final int LOC   = 0x2;

    /** Kind for eloc constraint. */
    public static final int ELOC  = 0x4;

    /** Kind for ltl synch constraint. */
    public static final int LTLSYNCH  = 0x8;

    /** Kind for one way ltl synch constraint. */
    public static final int LTLSYNCHIMPLY  = 0x10;

    ///////////////////////////////////////////////////////////////////
    ////                          public methods                   ////

    /** Add an event reference to the constraint.
     *  @param  e  the event reference need to be added.
     *  @see #getEvents()
     */
    public void addEvent(Event e) {
        Iterator events = _events.iterator();
        while (events.hasNext()) {
            Event event = (Event)events.next();
            if (event.equals(e)) {
                return;
            }
        }
        _events.addLast(e);
    }

    /** Add a pair of structural keyword and its resolved value to the
     * constraint. The types of values:
     * <ul>
     * <li>getconnectionnum - Integer
     * <li>getnthconnectionsrc - INode
     * <li>getnthconnectionport - IPort
     * </ul>
     *  @param keyword  the structural keyword.
     *  @param value  the resolved value of the keyword.
     *  @see #getStructureValues()
     */
    public void addStructureValue(String keyword, Object value) {
        if (_structureValues.containsKey(keyword)) {
            return;
        }
        _structureValues.put(keyword, value);
    }

    /** Get the container of the constraint.
     *  @return The network node that contains the constraint.
     *  @see #setContainer(Object)
     */
    public INode getContainer() {
        return _container;
    }

    /** Get the list of event references in the constraint.
     *  @return A linked list of events.
     *  @see #addEvent(Event)
     */
    public LinkedList getEvents() {
        return _events;
    }

    /** Get the formula text of the constraint.
     *  @return A string that contains the formula text of the constraint.
     *  @see #setFormula(String)
     */
    public String getFormula() {
        return _formula;
    }

    /** Get the index of the constraint.
     *  @return The index of the constraint.
     *  @see #setIndex(int)
     */
    public int getIndex() {
        return _index;
    }

    /** Get the type of the constraint.
     *  @return The type of the constraint.
     *  @see #setKind(int)
     */
    public int getKind() {
        return _kind;
    }

    /** Get the source code of the constraint.
     *  @return A string that contains the source code of the constraint.
     *  @see #setSource(String)
     */
    public String getSource() {
        return _source;
    }

    /** Get the hash table that stores all the structure keywords and
     *  values in the constraint.
     *  @return A hash table of (keyword, value) pairs.
     *  @see #addStructureValue(String, Object)
     */
    public Hashtable getStructureValues() {
        return _structureValues;
    }

    /** Set the container of the constraint.
     *  @param container  the container of the constraint
     *  @see #getContainer()
     */
    public void setContainer(Object container) {
        // FIXME: shouldn't the type of container be INode?
        if (container == null) {
            throw new RuntimeException("container was null?" + show());
        }
        _container = (INode) container;

    }

    /** Set the formula text of the constraint.
     *  @param formula The formula text of the constraint.
     *  @see #getFormula()
     */
    public void setFormula(String formula) {
        _formula = formula;
    }

    /** Set the index of the constraint.
     *  @param index The index of the constraint.
     *  @see #getIndex()
     */
    public void setIndex(int index) {
        _index = index;
    }

    /** Set the type of the constraint.
     *  @param kind The type of the constraint.
     *  @see #getKind()
     */
    public void setKind(int kind) {
        _kind = kind;
    }

    /** Set the source code of the constraint.
     *  @param source The source code of the constraint.
     *  @see #getSource()
     */
    public void setSource(String source) {
        _source = source.replace('\'', '"');
    }

    /** Get a String providing information about this constraint.
     *  @return A String providing information about this constraint.
     */
    public String show() {

        StringBuffer results = null;
        String indent = "          ";

        switch(_kind) {
        case LTL:
            results = new StringBuffer("LTL");
            break;
        case LOC:
            results = new StringBuffer("LOC");
            break;
        case ELOC:
            results = new StringBuffer("ELOC");
            break;
        case LTLSYNCH:
            results = new StringBuffer("LTL SYNCH");
            break;
        case LTLSYNCHIMPLY:
            results = new StringBuffer("LTL SYNCH IMPLY");
            break;
        default:
            throw new RuntimeException("Unexpected constraint type: "
                    + _kind );
            //break;
        }

        results.append(" Constraint (# " + _index + ")\n");
        results.append(indent + "o Source: " + _source + "\n");
        results.append(indent + "o Formula: " + _formula + "\n");
        results.append(indent + "o Container: "
                + _container.getName().toString() + "\n");

        if (_events.size() != 0) {
            results.append(indent + "o Event references: \n");
            Iterator events = _events.iterator();
            while (events.hasNext()) {
                Event event = (Event)events.next();
                results.append(indent + "  - " + event.show() + "\n");
            }
        } else {
            results.append(indent + "o No event references\n");
        }

        Enumeration values = _structureValues.elements();
        Enumeration keywords = _structureValues.keys();
        if (!keywords.hasMoreElements()) {
            results.append(indent + "o No structural values\n");
        } else {
            results.append(indent + "o Structural values: \n");
            while (keywords.hasMoreElements()) {
                String keyword = (String)keywords.nextElement();
                Object value = values.nextElement();
                if (value instanceof Integer)
                    results.append(indent + "  - " + keyword + " = "
                            + value.toString() + "\n");
                else if (value instanceof INode)
                    results.append(indent + "  - " + keyword + " = "
                            + ((INode)value).getName() + "\n");
                else if (value instanceof IPort)
                    results.append(indent + "  - " + keyword + " = "
                            + ((IPort)value).getName() + "\n");
            }
        }

        return  results.toString();
    }

    ///////////////////////////////////////////////////////////////////
    ////                        protected variables                ////

    /** The node where this constraint is defined. */
    protected INode _container;

    /** The kind of constraint. */
    protected int _kind;

    /** The sequence number(index) of this constraint in the container. */
    protected int _index;

    /** The source code for the constraint. */
    protected String _source;

    /** The formula text for the constraint used for checker generation.
     */
    protected String _formula;

    /** Mapping from structural keywords to their resolved values.
     *  For example: getconnectionnum(MediumObject, IfName) -> 4
     *  <br> getnthconnectionsrc(MediumObject, IfName, n) -> p1
     */
    protected Hashtable _structureValues = new Hashtable();

    /** Event references referred by this constraints. */
    protected LinkedList _events = new LinkedList();
}
