/* A one way ltl synch constraint in the elaborated network.

Metropolis: Design Environment for Heterogeneus Systems.

Copyright (c) 1998-2004 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software and that appropriate acknowledgments are made
to the research of the Metropolis group.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/


package metropolis.metamodel.runtime;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

//////////////////////////////////////////////////////////////////////////
//// LTLSynchImply
/**
 * A one way ltl synch constraint in the elaborated network, for example:
 * <p>
 * ltl synch(event1 || event2 || ...|| eventN => eventX: <br>
 v1@(event1, i) == v2@(event2, i), ...) <p>
 * or
 * <p>
 * ltl synch(eventX => event1 || event2 || ...|| eventN: <br>
 * v1@(event1, i) == vx@(eventx, i), ...).
 *
 *  @author   Xi Chen
 *  @version  $Id: LTLSynchImply.java,v 1.9 2004/09/07 20:41:31 cxh Exp $
 */
public class LTLSynchImply extends Constraint implements Serializable {

    ///////////////////////////////////////////////////////////////////
    ////                           constructors                    ////

    /**
     * Build a new ltl synch imply constraint in the network.
     * @param kind the constraint type LTLSYNCHIMPLY
     */
    public LTLSynchImply(int kind) {
        this(null, kind, -1);
    }


    /**
     * Build a new ltl synch imply constraint in the network.
     * @param container  the node which contains this constraint.
     * @param kind  the constraint type, LTLSYNCH.
     * @param index the sequence number of the constraint.
     */
    public LTLSynchImply(Object container, int kind, int index) {
        super(container, kind, index);
    }

    ///////////////////////////////////////////////////////////////////
    ////                          public methods                   ////

    /** Add an event at the left hand side of the imply operator.
     *  @param event an event at the left hand side of the imply operator.
     */
    public void addLeftEvent(Event event) {
        leftEvents.addLast(event);
    }

    /** Add an event at the right hand side of the imply operator.
     *  @param event an event at the right hand side of the imply operator.
     */
    public void addRightEvent(Event event) {
        rightEvents.addLast(event);
    }

    /** Transfer events in the event list to the left hand side of the
     *  imply operator.
     */
    public void eventsToLeftEvents() {
        leftEvents.addAll((Collection)_events);
        _events.clear();
    }

    /** Transfer events in the event list to the right hand side of the
     *  imply operator.
     */
    public void eventsToRightEvents() {
        rightEvents.addAll((Collection)_events);
        _events.clear();
    }

    /** Get the events at the left hand side of the imply operator.
     *  @return a list of events.
     */
    public LinkedList getLeftEvents() {
        return leftEvents;
    }

    /** Get the events at the right hand side of the imply operator.
     *  @return a list of events.
     */
    public LinkedList getRightEvents() {
        return rightEvents;
    }

    /** Add an equality var1@(event1, i) == var2@(event2, i).
     *  @param eq an equality
     */
    public void addEqualVars(EqualVars eq) {
        equalVars.addLast(eq);
    }

    /** Get the list of equalities.
     *  @return a list of EqualVars.
     */
    public LinkedList getEqualVars() {
        return equalVars;
    }

    /** Get a String providing information about this constraint.
     *  @return A String providing information about this constraint.
     */
    public String show() {
        StringBuffer results = new StringBuffer(super.show());
        String indent = "          ";

        results.append(indent + "o Imply: ");

        Iterator iter = leftEvents.iterator();
        while (iter.hasNext()) {
            Event e = (Event)iter.next();
            results.append(e.show());
            if (iter.hasNext())
                results.append("||");
        }

        results.append(" => ");
        iter = rightEvents.iterator();
        while (iter.hasNext()) {
            Event e = (Event)iter.next();
            results.append(e.show());
            if (iter.hasNext())
                results.append("||");
        }

        //if (equalVars.size() == 0)
        //return results.toString();

        results.append("\n");
        if (equalVars.size() == 0)
            results.append(indent + "o No equalities\n");
        else {
            results.append(indent + "o Equalities\n");
            iter = equalVars.iterator();
            while (iter.hasNext()) {
                EqualVars eq = (EqualVars)iter.next();
                results.append(indent + "  - ");
                results.append(eq.show());
                results.append("\n");
            }
        }

        return results.toString();
    }

    /** Check the consistency between the event list and the equalities.
     *  Add the events from the imply part to the central event list.
     */
    public void checkEvents() {
        Iterator iter = leftEvents.iterator();
        while (iter.hasNext())
            this.addEvent((Event)iter.next());

        iter = rightEvents.iterator();
        while (iter.hasNext())
            this.addEvent((Event)iter.next());

        if (false) {
            throw new RuntimeException("Error on ltl synch imply constraint: "
                    + "event references in equality part must appear in "
                    + "the event list.");
        }
    }

    ///////////////////////////////////////////////////////////////////
    ////              protected variables                ////

    /** The list of events on the left hand side of the imply operator. */
    protected LinkedList leftEvents = new LinkedList();

    /** The list of events on the right hand side of the imply operator. */
    protected LinkedList rightEvents = new LinkedList();

    /** A list of the equalities which are pairs in the form of
     *  var1@(event1, i) == var2@(event2, i).
     */
    protected LinkedList equalVars = new LinkedList();
}

